<?php
App::uses('BlogPluginAppController', 'BlogPlugin.Controller');
App::uses('CustomEmail', 'Lib');

class BlogCommentsController extends BlogPluginAppController
{
	public function beforeFilter()
	{
		$this->Auth->allow('post_comment');

		return parent::beforeFilter();
	}

	protected function _adminFormFields()
	{
		$fields = parent::_adminFormFields();
		unset($fields['blog_post_id'], $fields['parent_id']);
		$fields['comment']['type'] = 'text_plain';
		$fields['comment']['style'] = 'width: 360px;height:150px;';


		return $fields;
	}


	public function admin_delete($id)
	{
		$Model = $this->{$this->modelClass}->alias;

		$data = $this->BlogComment->findById($id);

		$this->$Model->delete($id);

		$this->Session->setFlash(array(
			'title' => 'Comment deleted',
			'description' => 'Comment has been successfully deleted!'
		), 'flash_success');

		return $this->redirect(
			array(
				'admin' => true,
				'plugin' => 'blog_plugin',
				'controller' => 'blog_posts',
				'action' => 'edit',
				$data['BlogComment']['blog_post_id']
			)
		);
	}

	public function admin_edit($id = null) {

		$Model = $this->{$this->modelClass}->alias;

		if (!empty($this->request->data)) {

			if ($this->$Model->saveAll($this->request->data, array('deep' => true))) {

				$action = ($id===null) ? 'created' : 'updated';

				$this->Session->setFlash(array(
					'title' => "Comment Saved",
					'description' => "Comment has been saved successfully",
				), 'flash_success');

				$data = $this->BlogComment->findById($id);
				return $this->redirect(
					array(
						'admin' => true,
						'plugin' => 'blog_plugin',
						'controller' => 'blog_posts',
						'action' => 'edit',
						$data['BlogComment']['blog_post_id']
					)
				);

			} else {

				$this->Session->setFlash(array(
					'title' => 'Save failed',
					'description' => 'Failed to save ' . InflectorExt::humanize($this->$Model->displayName) . ', see below for details'
				), 'flash_fail');
			}

		} elseif ($id !== null) {

			$this->request->data = $this->$Model->readForEdit($id);

			if (!$this->request->data) {

				throw new NotFoundException();

			}

		}

		$actionLabel = ($id===null) ? 'Add' : 'Edit';

		$this->set('title_for_layout', "$actionLabel " . InflectorExt::humanize($this->$Model->displayName));
		$this->set('fields', $this->_adminFormFields());
		$this->set('toolbar', $this->_adminFormToolbar($id));
		$this->set('imageSlots', $this->$Model->imageSlots);
		$this->set('documentSlots', $this->$Model->documentSlots);
		$this->set('menuFields', $this->_adminFormMenuFields(isset($this->request->data['Menu']['id']) ? $this->request->data['Menu']['id'] : null));
		$this->set('metaData', $this->_adminFormMetaFields());
		$this->_adminPopulateLookups();

		// Set the default template, this can be overridden in controllers using $this->view
		// or $this->render().
		$this->view = 'admin_form';

	}


	public function post_comment()
	{
		if ($this->request->is('post') && ! empty($this->request->data)) {

			if (isset($this->request->data['BlogComment']['email_confirm']) && ! empty($this->request->data['BlogComment']['email_confirm'])) {

				// spam field was field in, redirect them to the login page, will display success message
				$this->Session->setFlash(
					'Comment has been posted.',
					'BlogPlugin.message',
					array('class' => 'success'),
					'blog-comment'
				);

			} else {

				if (! isset($this->request->data['BlogComment']['parent_id']) || empty($this->request->data['BlogComment']['parent_id'])) {

					$this->request->data['BlogComment']['parent_id'] = null;
				}

				if (isset($this->request->data['BlogComment']['comment_id']) && empty($this->request->data['BlogComment']['comment_id'])) {

					if ($this->BlogComment->saveAll($this->request->data)) {

						$this->Session->setFlash(
							'Comment has been posted.',
							'BlogPlugin.message',
							array('class' => 'success'),
							'blog-comment'
						);

						$post = $this->BlogComment->BlogPost->findById($this->request->data['BlogComment']['blog_post_id']);

						// send email to admin informing of new comment
						$email = new CustomEmail('sendgrid');

						$email->theme('Site');
						$email->template('BlogPlugin.new-comment');
						$email->emailFormat('html');
						$email->to(Configure::read('SiteSetting.admin_email'));
						$email->from(array(Configure::read('SiteSetting.admin_email') => Configure::read('SiteSetting.site_title')));
						$email->subject('New blog comment posted');
						$email->viewVars(array(
							'post' => $post,
							'comment_id' => $this->BlogComment->id
						));
						$email->helpers(array(
								'Html'
						));
						$email->send();

					} else {

						$this->Session->setFlash(
							'There was an error saving your comment, please try again.',
							'BlogPlugin.message',
							array('class' => 'error'),
							'blog-comment'
						);
					}
				}
			}
		}

		$this->redirect(
			$this->referer() . '#blog-comments'
		);
	}

}