<?php
App::uses('BlogPluginAppController', 'BlogPlugin.Controller');
App::uses('CakeTime', 'Utility');

class BlogPostsController extends BlogPluginAppController {

    public $components = array('RequestHandler');

	public function beforeFilter() {

		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'view',
			'archives'
		));

	}

	protected function _adminFilterFields()
	{
		return array(
			'BlogPost.name' => array(
				'label' => 'Name',
				'type' => 'string',
				'compare' => array(
					'BlogPost.name LIKE' => '%%s%'
				)
			),
			'BlogPost.author' => array(
				'label' => 'Author',
				'type' => 'string',
				'compare' => array(
					'BlogPost.author LIKE' => '%%s%'
				)
			),
			'BlogPost.posted' => array(
				'label' => 'Posted',
				'type' => 'date',
				'compare' => array(
					'and' => array(
						'BlogPost.posted >=' => '%s 00:00:00',
						'BlogPost.posted <=' => '%s 24:00:00'
					)
				)
			)
		);
	}

	/**
	 * override the admin index columns
	 */
	protected function _adminIndexColumns()
	{
		$columns = array(
			'BlogPost.name' => array(
				'type' => 'string_full',
				'null' => true,
				'default' => null,
				'length' => (int) 100,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8',
				'label' => 'Name'
			),
			'BlogPost.author' => array(
				'type' => 'string',
				'null' => true,
				'default' => null,
				'length' => (int) 100,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8',
				'label' => 'Author'
			),
			'BlogCategory.name' => array(
				'type' => 'string_full',
				'null' => true,
				'default' => null,
				'length' => (int) 100,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8',
				'label' => 'Category'
			),
			'BlogPost.posted' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null,
				'label' => 'Posted On'
			),
			'BlogPost.is_active' => array(
				'type' => 'boolean',
				'null' => true,
				'default' => null,
				'length' => (int) 1,
				'label' => 'Active'
			)
		);

		return $columns;
	}

	protected function _adminIndexPaginate() {

		$conditions = $this->_processFilter();

		$paginate = array(
			'conditions'=>$conditions,
			'contain' => array(
				'BlogCategory'
			)
		);

		return $paginate;
	}


	protected function _adminFormFields() {

		return array(
			'BlogPost.id' => array(
				'type' => 'integer',
				'null' => false,
				'default' => null,
				'length' => (int) 11,
				'key' => 'primary'
			),
			'BlogPost.created' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null
			),
			'BlogPost.modified' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null
			),
			'BlogPost.name' => array(
				'type' => 'string',
				'null' => true,
				'default' => null,
				'length' => (int) 100,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8'
			),
			'BlogPost.author' => array(
				'type' => 'string',
				'null' => true,
				'default' => null,
				'length' => (int) 100,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8'
			),
			'BlogPost.posted' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null
			),
			'BlogPost.body' => array(
				'type' => 'html',
				'null' => true,
				'default' => null,
				'length' => null,
				'collate' => 'utf8_general_ci',
				'charset' => 'utf8'
			),
			'BlogPost.blog_category_id' => array(
				'type' => 'select',
				'label' => 'Category'
			),
			'BlogPost.tags' => array(
				'type' => 'string',
				'after' => '&nbsp;&nbsp;Seperate multiple tags with a comma.'
			),
			'BlogPost.is_active' => array(
				'type' => 'boolean',
				'null' => false,
				'default' => '0',
				'length' => (int) 1
			),
			'Route.id' => array(
				'type' => 'hidden'
			),
			'Route.alias' => array(
				'type' => 'hidden',
				'length' => (int) 128
			)
		);
	}

	/**
	 * Defines the buttons in the toolbar displayed on an admin_form
	 *
	 * Override in your controller to customise
	 */
	protected function _adminFormToolbar($id = null) {

		$Model = $this->{$this->modelClass}->alias;

		$actions = array(
			'Add New' => array(
				'url' => array('action' => 'add')
			)
		);

		$data = null;

		if ($id) {

			$data = $this->$Model->findById($id);
		}

		if (!isset($data[$Model]['is_protected']) || !$data[$Model]['is_protected']) {

			$actions['Delete'] = array(
				'url' => array('action' => 'delete', $id),
				'class' => 'toolbar__btn--delete'
			);
		}

		return $actions;
	}

	public function _adminPopulateLookups()
	{
		$blogCategories = array('0' => 'Uncategorised');

		$this->loadModel('BlogPlugin.BlogCategory');
		$categories = $this->BlogCategory->find('list');
		$blogCategories = $blogCategories + $categories;

		$this->set('blogCategories', $blogCategories);
	}

	public function admin_edit($id = null)
	{
		parent::admin_edit($id);
		$this->view = 'admin_edit';
	}


    /**
     * blog index
     *
     * display blog listing page
     */
    public function index()
    {
        // Fall back incase no page is found
        $this->set('title_for_layout', "Blog");

        $this->loadModel('Page');

        // see if there is a linked content page
        $page_id = Configure::read('BlogPlugin.page_id');

        if (! empty($page_id)) {
            $page = $this->Page->readForView($page_id);

            if (! empty($page)) {

                $this->Meta->set($page, 'Page');

                $this->set('page', $page);
            }
        }

        if ($this->RequestHandler->isRss() ) {

            $posts = $this->BlogPost->find('all', array(
                'conditions' => array(
                    'BlogPost.is_active' => 1,
                    'BlogPost.posted <' => date('Y-m-d H:i:s')
                ),
                'order' => 'BlogPost.posted DESC',
                'limit' => 20
            ));

            $description = strip_tags($page['Page']['body']);
            $desc_len = strlen($description);
            if ($desc_len > 0 && $desc_len >= 250) {

                $description = String::truncate($description, 250);

            } elseif ($desc_len == 0) {

                $description = $page['Page']['title'];
            }

            $channel = array(
                'title' => $page['Page']['title'],
                'link' => array(
                    'plugin' => 'blog_plugin',
                    'controller' => 'blog_posts',
                    'action' => 'index'
                ),
                'description' => $description,
                'language' => 'en-us'
            );

            return $this->set(compact('channel', 'posts'));
        }

        $this->paginate = array(
            'conditions' => array(
                'BlogPost.is_active' => 1,
                'BlogPost.posted <' => date('Y-m-d H:i:s')
            ),
            'order' => 'BlogPost.posted DESC',
            'contain' => array(
                'ListingImage',
                'BlogComment' => array(
                    'order' => 'BlogComment.created ASC'
                ),
                'BlogCategory'
            ),
            'limit' => 10
        );

        $this->set('data', $this->paginate());

        // call the widgets
        $this->widgetLatestPosts();
        $this->widgetCategories();
        $this->widgetArchives();
    }


	/**
	 * view individual blog post
	 */
	public function view($id)
	{
		$data = $this->BlogPost->readForView($id);
		$this->Meta->set($data);

		$this->set('data', $data);

		// call the widgets
		$this->widgetLatestPosts();
		$this->widgetCategories();
		$this->widgetArchives();
	}

	/**
	 * Blog archive, listing for blogfs from a certian month
	 *
	 * @param string $date given month to get posts from
	 */
	public function archives($date)
	{
		// Fall back incase no page is found
		$this->set('title_for_layout', "Blog");

		$date_bits = explode('-', $date);
		$date_short = $date_bits['0'] . '-' . $date_bits['1'];

		$category = array(
			'BlogArchive' => array(
				'name' => CakeTime::format('F Y', $date)
			)
		);

		$this->Meta->set($category, 'BlogArchive');
		$this->set('category', $category);

		$this->paginate = array(
			'conditions' => array(
				'BlogPost.is_active' => 1,
				'BlogPost.posted >=' => $date_short . '-01 00:00:00',
				'BlogPost.posted <=' => $date . ' 23:59:59',
				'BlogPost.posted <' => date('Y-m-d H:i:s')
			),
			'order' => 'BlogPost.posted DESC',
			'contain' => array(
				'ListingImage',
				'BlogComment',
				'BlogCategory'
			),
			'limit' => 10
		);

		$this->loadModel('BlogPlugin.BlogPost');
		$this->set('data', $this->paginate('BlogPost'));

		// call the widgets
		$this->widgetLatestPosts();
		$this->widgetCategories();
		$this->widgetArchives();

	}

}
